﻿--[[
	General Utility Functions	
]]


KarmaUtility = {
	SpecialCharsTable = {
		["\194\161"] = "!",	["\194\170"] = "a", ["\194\186"] = "o", ["\194\191"] = "?", ["\195\128"] = "A", ["\195\129"] = "A",
		["\195\130"] = "A", ["\195\131"] = "A", ["\195\133"] = "A", ["\195\135"] = "C", ["\195\136"] = "E", ["\195\137"] = "E",
		["\195\138"] = "E", ["\195\139"] = "E", ["\195\140"] = "I", ["\195\141"] = "I", ["\195\142"] = "I", ["\195\143"] = "I",
		["\195\144"] = "D", ["\195\145"] = "N", ["\195\146"] = "O", ["\195\147"] = "O", ["\195\148"] = "O", ["\195\149"] = "O",
		["\195\152"] = "O", ["\195\153"] = "U", ["\195\154"] = "U", ["\195\155"] = "U", ["\195\157"] = "Y", ["\195\160"] = "a",
		["\195\161"] = "a", ["\195\162"] = "a", ["\195\163"] = "a", ["\195\165"] = "a", ["\195\167"] = "c", ["\195\168"] = "e",
		["\195\169"] = "e", ["\195\170"] = "e", ["\195\171"] = "e", ["\195\172"] = "i", ["\195\173"] = "i", ["\195\174"] = "i",
		["\195\175"] = "i", ["\195\176"] = "d", ["\195\177"] = "n", ["\195\178"] = "o", ["\195\179"] = "o", ["\195\180"] = "o",
		["\195\181"] = "o", ["\195\184"] = "o", ["\195\185"] = "u", ["\195\186"] = "u", ["\195\187"] = "u", ["\195\189"] = "y",
		["\195\191"] = "y", ["\195\132"] = "Ae", ["\195\134"] = "AE", ["\195\150"] = "Oe", ["\195\156"] = "Ue", ["\195\158"] = "TH",
		["\195\159"] = "ss", ["\195\164"] = "ae", ["\195\166"] = "ae", ["\195\182"] = "oe", ["\195\188"] = "ue", ["\195\190"] = "th"
	}
};


function KarmaUtility:ToStr( val )
	if not val then
		val = "[nil]"
	elseif type(val)=="table" then
		val = "[table]"
	elseif type(val)=="function" then
		val = "[function]"
	end
	return val
end


function KarmaUtility:Print(msg, r, g, b)
	r = r or 1; g = g or 1; b = b or 0;
	DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
end


function KarmaUtility:SysMsg( ... )
	local msg = ""
	for i=1, select("#", ...), 1 do
		msg = msg .. tostring( self:ToStr( select(i, ...) ) );
  	end
    self:Print( msg, 1, 0.5, 0 );
end


function KarmaUtility:Debug( ... )
    if ( self._store.db and self._store.db.Options and self._store.db.Options.DebugMode == 1 ) then
    	self:SysMsg( "  #", ..., "#" );
    end
end


function KarmaUtility:FixZero( n )
    if n < 10 then n = "0" .. n; end
    return n
end


function KarmaUtility:StripSpecialChars( s )
	for k,v in pairs(self.SpecialCharsTable) do
		s = string.gsub(s, k, v);	
	end
	return s;
end


function KarmaUtility:ConvertGlobalString( s )
	-- Stolen from nurfed (and fixed for german clients)
	s = string.gsub(s, "%%%d%$", "%%"); 
	s = string.gsub(s, "%%s", "(.+)");
	s = string.gsub(s, "%%d", "(%%d+)");
	return s;
end


function KarmaUtility:TableGetRaw( t, key )
	if not t or type(t) ~= "table" then return nil end
	for i, v in pairs( t ) do
		if i == key then return v end
	end
	return nil;	
end


function KarmaUtility:TableAddUnique( t, key )
	if not t or type(t) ~= "table" then return end
	local found = false
	for i, v in pairs ( t ) do
		if v == key then found = true; break; end
	end 
	if not found then table.insert(t,key)end
end


-- dump table inverited values
function KarmaUtility:TableDump( t, name )
	if name then self:SysMsg(name,":"); end
	if not t then return end
	local a = { t };
	for i = 1, 10 do 
		local m = getmetatable( a[i] );
		if m and m.__index and type(m.__index) == "table" then table.insert(a,m.__index) else break end		
	end	
	local idx = { };
	for i2, v2 in pairs( a ) do
		for i, v in pairs( v2 ) do
			self:TableAddUnique( idx, i )
		end
	end 
	table.sort( idx );
	for i, v in pairs( idx ) do
		local s = "";
		for i2, v2 in pairs( a ) do
			s = s .. ":" .. tostring(self:ToStr(self:TableGetRaw(v2, v)));
		end	
		self:SysMsg( "    ", v, "=", s );
	end
end

function KarmaUtility:sortedpairs(t,comparator)
	local sortedKeys = {};
	table.foreach(t, function(k,v) table.insert(sortedKeys,k) end);
	table.sort(sortedKeys,comparator);
	local i = 0;
	local function _f(_s,_v)
		i = i + 1;
		local k = sortedKeys[i];
		if (k) then
			return k,t[k];
		end
	end
	return _f,nil,nil;
end


function KarmaUtility:GetTimestamp( s )
    if not s then return nil; end
    local _, _, mon, day, year, hr, min, sec = string.find(s, "(%d+)/(%d+)/(%d+) (%d+):(%d+):(%d+)");
    local table = date("*t", time());
    local timestamp;
    table["month"] = tonumber(mon);
    table["year"] = tonumber("20" .. year);
    table["day"] = tonumber(day);
    table["hour"] = tonumber(hr);
    table["min"] = tonumber(min);
    table["sec"] = tonumber(sec);
    timestamp = time(table);
    return timestamp;
end


function KarmaUtility:ColorToRGB( s )
    s = strlower(strsub(s, 3));
    local tbl = { };
    tbl[1], tbl[2], tbl[3], tbl[4], tbl[5], tbl[6] = strsub(s, 1, 1), strsub(s, 2, 2), strsub(s, 3, 3), strsub(s, 4, 4), strsub(s, 5, 5), strsub(s, 6, 6);
    
    local highvals = { ["a"] = 10, ["b"] = 11, ["c"] = 12, ["d"] = 13, ["e"] = 14, ["f"] = 15 };
    for k, v in pairs(tbl) do
        if ( highvals[v] ) then
            tbl[k] = highvals[v];
        elseif ( tonumber(v) ) then
            tbl[k] = tonumber(v);
        end
    end
    local r, g, b = (tbl[1]*16+tbl[2])/255, (tbl[3]*16+tbl[4])/255, (tbl[5]*16+tbl[6])/255;
    if ( not r or r > 1 or r < 0 ) then
        r = 1;
    end
    if ( not g or g > 1 or g < 0 ) then
        g = 1;
    end
    if ( not b or b > 1 or b < 0 ) then
        b = 1;
    end
    return r, g, b;
end


function KarmaUtility:CopyDeep(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end


function KarmaUtility:MergeDeep(dst, src)
 	local function _canCopy(dst, src)
 		if not dst then	return true	end
 	 	if (not src) or
 	 		(type(dst) == "table" and type(src) ~= "table") or 
 	 		(type(dst) ~= "table" and type(src) == "table")  then
 			return false
 		end
	 	return true
	end
	local function _copy(dst, src)
		for i, v in pairs(src) do
			if _canCopy(dst[i], v) then
				if type(v) ~= "table" then 
					if dst[i] ~= v then dst[i] = v end
				else
					if not dst[i] then dst[i] = { } end
					_copy(i, dst[i], v) 
				end
			end 	
		end
		return dst
	end
	if _canCopy(dst, src) then 
		if type(src) ~= "table" then return src end
		if not dst then dst = { } end 	
	 	_copy(dst, src)
	end
	return dst 
end

